pub enum SqliteError {
TransactionMissing,
Sqlite(Error),
Migrate(MigrateError),
Encode(String, EncodeError),
Decode(String, DecodeError),
}Available on crate feature
sqlite only.Expand description
Error when interacting with a SQLite store implementation.
Variants
TransactionMissing
This is a critical error as it indicates that something is wrong with the usage of this API: Queries using transactions can only ever occur if a transaction was started before.
Sqlite(Error)
SQLite database and connection error.
Migrate(MigrateError)
SQL table schema migration error.
Encode(String, EncodeError)
An I/O error occurred while encoding bytes before storing them into the database. This is a critical error.
Decode(String, DecodeError)
Invalid, corrupted data was found in the database. This is a critical error.