Module sqlite

Module sqlite 

SQLite database implementation with associated utility functions.

Structs

SqliteStore
SQLite database with connection pool and transaction provider.
SqliteStoreBuilder
Builder for SqliteStore.
TransactionPermit
Locked context marking the lifetime of a single transaction.

Enums

DecodeError
Error decoding value retrieved from a store.
SqliteError
Error when interacting with a SQLite store implementation.

Functions

connection_pool
Creates the SQLite connection pool.
create_database
Creates the SQLite database if it doesn’t already exist.
drop_database
Drops the SQLite database if it exists.
migrations
Gets migrations from folder without running them.
run_pending_migrations
Runs any pending database migrations from inside the application.

Type Aliases

SqlitePool
Sqlite connection pool.
Transaction
An in-progress database transaction.