SQLite database implementation with associated utility functions.
Structs
- Sqlite
Store - SQLite database with connection pool and transaction provider.
- Sqlite
Store Builder - Builder for
SqliteStore. - Transaction
Permit - Locked context marking the lifetime of a single transaction.
Enums
- Decode
Error - Error decoding value retrieved from a store.
- Sqlite
Error - Error when interacting with a SQLite store implementation.
Functions
- connection_
pool - Creates the SQLite connection pool.
- create_
database - Creates the SQLite database if it doesn’t already exist.
- drop_
database - Drops the SQLite database if it exists.
- migrations
- Gets migrations from folder without running them.
- run_
pending_ migrations - Runs any pending database migrations from inside the application.
Type Aliases
- Sqlite
Pool - Sqlite connection pool.
- Transaction
- An in-progress database transaction.