pub struct SqliteStore { /* private fields */ }sqlite only.Expand description
SQLite database with connection pool and transaction provider.
This struct can be cloned and used in multiple places in the application. Every cloned instance
will re-use the same connection pool and have access to the same transaction instance if one
was started. To guard against sharing transactions unknowingly across unrelated database
queries, a concept of a TransactionPermit was introduced which does not protect from misuse
but helps to make “holding” a transaction explicit.
Please note that SQLite strictly serializes transactions with writes and will block any
parallel attempt to begin another one. Processes starting a transaction will acquire a
TransactionPermit and keep it until the transaction was committed or rolled back. If the
query only involves reads it is recommended to not use transactions and use the execute
method directly as acquiring transactions will potentially block other processes to do work.
Design decisions
This storage API design was chosen to make the dynamics of the underlying SQLite database explicit to avoid potentially introducing subtle bugs. Internally any process can access the transaction object to do writes and (uncommitted) reads (see “Transaction I” in diagram). Care is required when designing systems like that as it’s still possible to allow concurrent processes to read and write within the same transaction (for example one process could roll back the transaction while the other one assumed it will be committed). Usually developers want to design writes to the database within a transaction if they need consistency and atomicity guarantees. “Unrelated” queries can be “pooled” in one transaction (for performance reasons for example) if consistency is guaranteed by all involved processes and the underlying data-model (see “Transaction II” in diagram).
Transaction I:
begin ---------------------> commit
Process I:
--> write --> read -->
Transaction II:
begin ----------------------> commit
Process II:
--> write --> write -->
Process III:
--> read --> write --->Another design decision is to not expose transactions to the high-level storage APIs (similar
to the “Repository Pattern”). Users of the storage methods like get_operation (in
OperationStore) etc. do not need to explicity deal with transaction objects, as this is
handled internally now. Like this it is possible to separate the “logic” from the “storage”
layer and keep the code clean.
Implementations
Sourceimpl SqliteStore
impl SqliteStore
Sourcepub fn from_pool(pool: SqlitePool) -> Self
pub fn from_pool(pool: SqlitePool) -> Self
Creates a new SqliteStore using the provided connection pool.
Sourcepub fn pool(&self) -> &SqlitePool
pub fn pool(&self) -> &SqlitePool
Returns a reference to the connection pool.
Sourcepub async fn temporary() -> Self
Available on crate features test_utils only.
pub async fn temporary() -> Self
test_utils only.Builds an in-memory SQLite database with a randomised name for testing purposes.
Sourcepub async fn tx<F, R>(&self, f: F) -> Result<R, SqliteError>
pub async fn tx<F, R>(&self, f: F) -> Result<R, SqliteError>
Executes a SQL query within a transaction.
This method will return an error when no transaction is currently given. Make sure to call
begin before.
If the query fails the user probably wants to roll back the transaction and free the permit. This is not handled automatically.
Sourcepub async fn execute<F, R>(&self, f: F) -> Result<R, SqliteError>
pub async fn execute<F, R>(&self, f: F) -> Result<R, SqliteError>
Executes a SQL query directly.